// THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
// (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
// WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY AND
// RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
// NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE
// FOR DAMAGES OF ANY KIND RESULTING FROM ITS USE.  Without limiting
// the generality of the foregoing, neither the author, nor JAIR, nor
// JAIR's publishers and distributors, warrant that the Source Code
// will be error-free, will operate without interruption, or will meet
// the needs of the user.

#include "pdb.h"

// Initialize the dag.  Each variable gets its own node at the outset.

void Dag::resize(size_t s)
{
  vector<Node>::resize(s);
  for (size_t i = 0 ; i < s ; ++i) {
    (*this)[i].clear();
    followers(i).insert(i);
    preceders(i).insert(i);
  }
}

// add an arc to a Dag.  Just walking through it, really.  For each
// preceder p of a, add b's followers to p's.  (If b is already a
// follower of p, we can skip this.)  Followers of b are dual.

void Dag::add_arc(atom a, atom b)
{
  if (leq(a,b)) return;           // nothing to do
  for (auto p : preceders(a))
    if (!followers(p).includes(b)) followers(p) |= followers(b);
  for (auto f : followers(b))
    if (!preceders(f).includes(a)) preceders(f) |= preceders(a);
}

// weakening is similarly easy.  All the followers of a lose any
// internal information.  So all such followers have only themselves
// as followers, and preceders that lose all of a's followers.

void Dag::weaken(atom a)
{
  VarSet later = proper_followers(a);
  for (auto l : later) {
    followers(l).clear();
    followers(l).insert(l);
    preceders(l) -= later;
    preceders(l).insert(l);
  }
}
