// THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
// (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
// WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY AND
// RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
// NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE
// FOR DAMAGES OF ANY KIND RESULTING FROM ITS USE.  Without limiting
// the generality of the foregoing, neither the author, nor JAIR, nor
// JAIR's publishers and distributors, warrant that the Source Code
// will be error-free, will operate without interruption, or will meet
// the needs of the user.

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "solver.h"

// Comparison based on VSIDS

bool mg_heap::cmp(atom i, atom j) const
{
  return slv->bvalue[i] > slv->bvalue[j];
}

static inline int parent(int x) { return (x - 1) >> 1; }

void mg_heap::move_up(int idx)
{
  atom a = (*this)[idx];
  for (int p = parent(idx) ; p >= 0 && cmp(a,(*this)[p]) ; p = parent(p)) {
    (*this)[idx] = (*this)[p];
    location[(*this)[p]] = idx;
    idx = p;
  }
  (*this)[idx] = a;
  location[a] = idx;
}

void mg_heap::move_down(int idx)
{
  atom a = (*this)[idx];  
  while (2 * idx + 1 < (int) size()) {
    int child = 2 * idx + 1;
    int bigger_child = child +
      (child + 1 < (int) size() && cmp((*this)[child + 1],(*this)[child]));
    if (!cmp((*this)[bigger_child],a)) break;
    (*this)[idx] = (*this)[bigger_child];
    location[(*this)[idx]] = idx;
    idx = bigger_child;
  }
  (*this)[idx] = a;
  location[a] = idx;
}

void mg_heap::clear()
{
  vector<atom>::clear();
  fill(location.begin(),location.end(),-1);
}

// Add a scored element to an mg_heap
void mg_heap::insert(atom v)
{  
  if (location.size() < v + 1) location.resize(v + 1,-1);
  location[v] = size();
  push_back(v);
  move_up(location[v]);
}

// Return the top of the mg_heap (and pop it off)
atom mg_heap::front() 
{
  atom ans = (*this)[0];
  (*this)[0] = back();
  location[(*this)[0]] = 0;     // don't pop yet in case queue is empty
  location[ans] = -1;
  pop_back();
  if (size() > 1) move_down(0);
  return ans;
}

//  Rebuild the heap from scratch

void mg_heap::rebuild(const vector<atom> &v)
{
  clear();
  for (size_t i = 0; i < v.size(); i++) {
    location[v[i]] = i;
    push_back(v[i]); 
  }
  for (int i = size() / 2 - 1; i >= 0; i--) move_down(i);
}
