// THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
// (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
// WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY AND
// RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
// NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE
// FOR DAMAGES OF ANY KIND RESULTING FROM ITS USE.  Without limiting
// the generality of the foregoing, neither the author, nor JAIR, nor
// JAIR's publishers and distributors, warrant that the Source Code
// will be error-free, will operate without interruption, or will meet
// the needs of the user.

#ifndef __PDB__
#define __PDB__

#include "stl.h"

// Node in the dag.  Very simple: just all following atoms, all
// preceding atoms. The atom itself should be in each set.

struct VarSet : vector<atom>    // sorted vector, basically
{
  bool   includes(atom a) const;
  void   insert  (atom a);
  VarSet operator &(const VarSet &other) const;
  VarSet operator -(const VarSet &other) const;
  VarSet operator |(const VarSet &other) const;

  VarSet operator &=(const VarSet &other) { return *this = *this & other; }
  VarSet operator -=(const VarSet &other) { return *this = *this - other; }
  VarSet operator |=(const VarSet &other) { return *this = *this | other; }
};

class Node
{
  VarSet followers, preceders;

  void clear() { followers.clear(); preceders.clear(); }
  bool empty() const { return followers.empty(); }

  friend class Dag;
};

class Dag : public vector<Node>
{
 public:
  bool lt          (atom x, atom y) const { return leq(x,y) && !leq(y,x); }
  bool gt          (atom x, atom y) const { return lt(y,x); }
  bool leq         (atom x, atom y) const { return followers(x).includes(y); }
  bool geq         (atom x, atom y) const { return leq(y,x); }
  bool equiv       (atom x, atom y) const { return leq(x,y) && leq(y,x); }
  bool incomparable(atom x, atom y) const { return !leq(x,y) && !leq(y,x); }

 private:
  VarSet       & followers(atom n)       { return (*this)[n].followers; }
  VarSet       & preceders(atom n)       { return (*this)[n].preceders; }
 public:
  const VarSet & followers(atom n) const { return (*this)[n].followers; }
  const VarSet & preceders(atom n) const { return (*this)[n].preceders; }

  VarSet         equiv_class(atom n) const 
    { return followers(n) & preceders(n); }
  VarSet         proper_followers(atom n) const
    { return followers(n) - preceders(n); }

  void resize         (size_t s);
  void add_arc        (atom a, atom b);
  void weaken         (atom a);

  friend ostream &operator<<(ostream &os, const Dag &d);
};

ostream &operator<<(ostream &os, const Dag &d);

#endif
