// THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
// (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
// WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY AND
// RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
// NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE
// FOR DAMAGES OF ANY KIND RESULTING FROM ITS USE.  Without limiting
// the generality of the foregoing, neither the author, nor JAIR, nor
// JAIR's publishers and distributors, warrant that the Source Code
// will be error-free, will operate without interruption, or will meet
// the needs of the user.

#include "solver.h"

// Move a single clause from one memory block to a fresh one.  But it
// may already have been moved, in which case it will have an entry in
// relocs telling you where it went.  When you're done, copy the
// offset in the new pool onto the incoming clause_id.

void pool::reloc(clause_id &cr, vector<clause_id> &relocs, pool& to)
{
  if (relocs[cr.index]) { cr = relocs[cr.index]; return; }
  clause& c = (clause &) (*this)[cr.index];
  relocs[cr.index] = to.size(); // set this first; cr gets destroyed next
  cr = to.size();
  to.alloc(c,c.learned);
  ((clause &)to[cr.index]).value = c.value;
}

// Copy all the clauses to a new memory pool.  Just go through one at
// a time: clauses, learned clauses, watchers, reasons and the
// bias_clause (if any).

void solver::reloc_all(pool& to)
{
  vector<clause_id> relocs(litpool.size(),-1);
  for (size_t i = 0; i < clauses.size(); i++)
    litpool.reloc(clauses[i],relocs, to);
  for (size_t i = 0; i < learned.size(); i++)
    litpool.reloc(learned[i],relocs, to);

  for (size_t v = 0; v < size(); v++) {
    for (auto &w : watches[int(literal(v,false))]) litpool.reloc(w,relocs,to);
    for (auto &w : watches[int(literal(v,true))]) litpool.reloc(w,relocs,to);
  }

  for (auto l : stack) 
    if (reason[l] && (relocs[reason[l].index] || is_reason(reason[l])))
      litpool.reloc(reason[l],relocs,to);

  if (bias_clause) litpool.reloc(bias_clause,relocs,to);
}

void solver::add_to_gamma(clause_id cid)
{
  gamma.insert(cid);
  for (auto l : *cid) goccur(l).push_back(cid);
}

void solver::recompute_gamma()
{
  gamma.clear();
  for (auto &i : goccurs) i.clear();
  for (auto i : clauses) if (i->gamma) add_to_gamma(i);
  for (auto i : learned) if (i->gamma) add_to_gamma(i);
}

// Garbage collect.  Just make a new pool that is barely big enough to
// hold what's left in the current one.

void solver::garbage_collect()
{
  pool to(litpool.size() - litpool.free); 
  reloc_all(to);
  litpool = to;
  recompute_gamma();
}
