// THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
// (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
// WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY AND
// RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
// NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE
// FOR DAMAGES OF ANY KIND RESULTING FROM ITS USE.  Without limiting
// the generality of the foregoing, neither the author, nor JAIR, nor
// JAIR's publishers and distributors, warrant that the Source Code
// will be error-free, will operate without interruption, or will meet
// the needs of the user.

#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <iostream>

using namespace std;

double get_cpu_time() {
  struct rusage ru;
  getrusage(RUSAGE_SELF, &ru);
  return (double)ru.ru_utime.tv_sec + (double)ru.ru_utime.tv_usec / 1000000; }

#if defined(__APPLE__)
#include <malloc/malloc.h>

double mem_used() {
  malloc_statistics_t t;
  malloc_zone_statistics(NULL, &t);
  return (double)t.max_size_in_use / (1024*1024); }
#else
double mem_used() { return 0; }
#endif

// Limit some CPU resource

void set_limit(int resource, rlim_t limit, const string &desc)
{
  rlimit rl;
  getrlimit(resource,&rl);
  if (rl.rlim_max == RLIM_INFINITY || limit < rl.rlim_max) {
    rl.rlim_cur = limit;
    if (setrlimit(resource,&rl) == -1)
      cerr << "WARNING: cannot set resource limit for " << desc << endl;
  }
}
