// THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
// (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
// WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY AND
// RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
// NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE
// FOR DAMAGES OF ANY KIND RESULTING FROM ITS USE.  Without limiting
// the generality of the foregoing, neither the author, nor JAIR, nor
// JAIR's publishers and distributors, warrant that the Source Code
// will be error-free, will operate without interruption, or will meet
// the needs of the user.

#include "pdb.h"

// VarSets are just ordered vectors.  But they should be pretty small
// most of the time, so we don't use sets or the sorted vector stuff.
// We just treat them as sorted vectors and do linear walks.  That
// seems to be faster than the obvious alternatives.

bool VarSet::includes(atom a) const
{
  return find(begin(),end(),a) != end();
}

void VarSet::insert(atom a)
{
  for (size_t i = 0 ; i < size() ; ++i)
    if (a < (*this)[i]) {
      push_back(back());
      for (size_t j = size() - 1 ; j > i ; --j) (*this)[j] = (*this)[j - 1];
      (*this)[i] = a;
      return;
    }
    else if (a == (*this)[i]) return;
  push_back(a);
}

VarSet VarSet::operator&(const VarSet &other) const
{
  VarSet ans;
  size_t i = 0, j = 0;
  while (i < size() && j < other.size()) {
    if ((*this)[i] == other[j]) {
      ans.push_back((*this)[i++]);
      ++j;
    }
    else if ((*this)[i] < other[j]) ++i;
    else ++j;
  }
  return ans;
}

VarSet VarSet::operator-(const VarSet &other) const
{
  VarSet ans;
  size_t i = 0, j = 0;
  while (i < size() && j < other.size()) {
    if ((*this)[i] == other[j]) { ++i; ++j; }
    else if ((*this)[i] < other[j]) ans.push_back((*this)[i++]);
    else ++j;
  }
  while (i < size()) ans.push_back((*this)[i++]);
  return ans;
}

VarSet VarSet::operator|(const VarSet &other) const
{
  VarSet ans;
  size_t i = 0, j = 0;
  while (i < size() && j < other.size()) {
    if ((*this)[i] == other[j]) {
      ans.push_back((*this)[i++]);
      ++j;
    }
    else if ((*this)[i] < other[j]) ans.push_back((*this)[i++]);
    else ans.push_back(other[j++]);
  }
  while (i < size()) ans.push_back((*this)[i++]);
  while (j < other.size()) ans.push_back(other[j++]);
  return ans;
}
