// THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
// (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
// WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY AND
// RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
// NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE
// FOR DAMAGES OF ANY KIND RESULTING FROM ITS USE.  Without limiting
// the generality of the foregoing, neither the author, nor JAIR, nor
// JAIR's publishers and distributors, warrant that the Source Code
// will be error-free, will operate without interruption, or will meet
// the needs of the user.

#include "solver.h"

// Watch the first two lits in a clause, recording the other lit as
// well.  Just push the appropriate watcher in the appropriate place.
// At the end, increment the total number of learned or original lits.

void solver::watch_clause(const clause_id ci) 
{
  const clause& c = *ci;
  if (c.size() < 2) return;
  watch(-c[0]).push_back(watcher(ci,c[1]));
  watch(-c[1]).push_back(watcher(ci,c[0]));
  (c.learned? learned_lits : clause_lits) += c.size();
}

// Unwatching a clause is the reverse.  The utility removes a watcher
// from a vector of such.  In general, it appears to be better to keep
// the watcher list ordered rather than swap the given watcher out for
// the end of the list.

static inline void remove(vector<watcher>& ws, const watcher& t)
{
  auto i = find(ws.begin(),ws.end(),t);
  copy(i + 1,ws.end(),i);
  ws.pop_back();
}

void solver::unwatch_clause(const clause_id ci) 
{
  const clause& c = *ci;
  if (c.size() < 2) return;
  remove(watch(-c[0]),watcher(ci,c[1]));
  remove(watch(-c[1]),watcher(ci,c[0]));
  (c.learned? learned_lits : clause_lits) -= c.size();
}
