// THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
// (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
// WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY AND
// RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
// NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE
// FOR DAMAGES OF ANY KIND RESULTING FROM ITS USE.  Without limiting
// the generality of the foregoing, neither the author, nor JAIR, nor
// JAIR's publishers and distributors, warrant that the Source Code
// will be error-free, will operate without interruption, or will meet
// the needs of the user.

#include "solver.h"

// Unit propagate (general).  Just propagate until there is nothing
// left, retaining the last conflict clause.

clause_id solver::unit_prop()
{
  clause_id conf;

  while (propagation_point < stack.size()) {
    clause_id cid = unit_prop(stack[propagation_point++]);
    if (cid) conf = cid;
    ++propagations;
  }
  return conf;
}

// Unit propagation from a single literal, returning a (possibly null)
// conflict clause.

// The work, of course, is in the watched literals.  The way it works
// is that we keep things so that the watchers for any particular
// clause are always the first two literals in the clause.  The
// watcher also indicates a specific other watched literal that we
// check first to see if it's satisfied (because then we don't have to
// do anything at all.)  

// If we are unit propagating from p so that -p is either c[0] or c[1]
// for a clause c, the first thing we do is swap c[1] and c[0] if
// needed so that c[1] == -p.

// Now if the second watched literal is true, we can stop, but as we
// do so, we update the "other" literal to be this other watched
// literal, so that we have a shot at benefitting later (even if the
// watched literals have been shuffled around a bit).

// As we do all of this, the set of clauses watching p needs to be
// rebuilt.  We build the new set in place, exploiting the fact that
// since -p is now false, the watched set can only get smaller.  So,
// for example, in the case where the other lit is satisfied, we just
// keep this clause on the list of clauses where p is watched, and
// move on.

clause_id solver::unit_prop(literal p)
{
  vector<watcher> &ws = watch(p);
  auto i = ws.begin();
  auto j = i;

  while (i != ws.end()) {
    if (is_true(i->other)) { *j++ = *i++; continue; } // copy ptr & you're done
    clause_id cr = *i++;
    clause   &c  = *cr;
    if (c[0] == -p) swap(c[0],c[1]); // c[1] is false
    watcher w(cr,c[0]);
    if (is_true(c[0])) { *j++ = w; continue; }

    for (size_t k = 2 ; k < c.size() ; k++) // find new watcher
      if (not_false(c[k])) {
        swap(c[1],c[k]);
        watch(-c[1]).push_back(w);
        goto nextw; 
      }

    *j++ = w;                   // no watcher; clause is unit or unsat
    if (is_false(c[0])) {       // unsat
      propagation_point = stack.size(); // clear out remaining propagations
      while (i < ws.end()) *j++ = *i++; // other watchers are still ok
      ws.resize(j - ws.begin()); // remove clauses no longer watched
      return cr;
    }
    else make_true(c[0],cr);    // unit -- make variable true
  nextw:;
  }
  ws.resize(j - ws.begin());    // remove clauses no longer watched
  return clause_id();
}
